/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package freemind.controller;

import freemind.main.FreeMind;
import freemind.view.mindmapview.MapView;
import freemind.view.mindmapview.NodeView;
import freemind.main.Tools;
import java.util.Vector;
import javax.swing.JOptionPane;//for testing purposes
import javax.swing.JColorChooser;
import javax.swing.JFileChooser;
import javax.swing.JTextField;
import javax.swing.JFrame;
import javax.swing.JViewport;
import freemind.model.simplemodel.NodeModel;//for testing purposes
import freemind.model.simplemodel.MapModel;
import freemind.model.MindMap;
import freemind.model.MindMapNode;
import java.awt.Component;
import java.awt.Color;
import java.io.File;
import javax.swing.AbstractAction;//Experimental
import javax.swing.Action;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ImageIcon;
import javax.swing.filechooser.FileFilter;
import java.awt.Point;
//for debugging purposes
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusAdapter;



/**
 * Provides the methods to edit/change a Node.
 * Forwards all messages to MapModel(editing) or MapView(navigation).
 */
public class Controller {

    private MapView view;
    private MindMap model;
    private JFrame frame;
    private PopupMenu popupMenu;
    public Action quit = new QuitAction();
    public Action newMap = new NewMapAction(this);
    public Action open = new OpenAction(this);
    public Action saveAs = new SaveAsAction(this);
    public Action save = new SaveAction(this);
    public Action cut = new CutAction(this);
    public Action paste = new PasteAction(this);
    public Action about = new AboutAction();
    public Action background = new BackgroundAction();
    public Action fork = new ForkAction();
    public Action bubble = new BubbleAction();
    public Action nodeColor = new NodeColorAction();
    public Action edgeColor = new EdgeColorAction();
    public Action linear = new LinearAction();
    public Action bezier = new BezierAction();
    public Action italic = new ItalicAction(this);
    public Action bold = new BoldAction(this);
    public Action underlined = new UnderlinedAction(this);
    public Action normalFont = new NormalFontAction(this);
    public FocusListener focusListener = new FocusListenerImpl();//debugging
    
    //
    // Constructors
    //

    public Controller() {
	//Create the Popup Menu
	popupMenu = new PopupMenu(this);
    }

    //
    // Initialization
    //

    public void setView(MapView view) {
	this.view = view;
    }

    public void setModel(MindMap model) {
	this.model = model;
    }

    public void setFrame(JFrame frame) {
	this.frame = frame;
    }

    //
    // Navigation
    //

    public void moveUp() {
	view.moveUp();
    }

    public void moveDown() {
	view.moveDown();
    }

    public void moveLeft() {
	view.moveLeft();
    }

    public void moveRight() {
	view.moveRight();
    }

    public void moveToRoot() {
	view.moveToRoot();
    }

    public void select( NodeView node ) {
	view.select(node);
    }

    public void centerNode() {
	view.centerNode();
    }

    //
    // Editing
    //

    public void addNew(NodeView parent) {
	NodeModel newNode = new NodeModel("New Node");
	model.insertNodeInto(newNode,parent.getModel(), 0 );
	edit(newNode.getViewer(),parent);
    }

    public void delete(NodeView node) {
	if (!node.isRoot()) {
	    model.removeNodeFromParent(node.getModel());
	}
    }

    public void edit(final NodeView node) {
	edit(node,node);
    }

    public void edit(final NodeView node,final NodeView toBeSelected) {
	view.scrollNodeToVisible(node);
	final JTextField input = new JTextField(node.getModel().toString());
	Point loc = node.getLocation();
	Point scroll = ((JViewport)view.getParent()).getViewPosition();
	Point content = frame.getContentPane().getLocation();
	//Todo: Remove the constant(40) (the menubar) from this calc
	input.setBounds(loc.x-scroll.x+content.x,loc.y-scroll.y+content.y+40,input.getPreferredSize().width,15);
	input.selectAll();
	input.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    view.select(toBeSelected);
		    //Focus is lost, so focusLost() is called, which does the work
		}
	    });
	input.addFocusListener(new FocusAdapter() {
		public void focusLost(FocusEvent e) {
  		    model.changeNode(node.getModel(),input.getText());
  		    frame.getLayeredPane().remove(input);
		}
	    });
		
	frame.getLayeredPane().add(input,2000);
	input.requestFocus();
	frame.repaint();
    }

    public void setFontSize(int fontSize) {
	model.setFontSize(view.getSelected().getModel(),fontSize);
    }

    public void setFont(String font) {
	model.setFont(view.getSelected().getModel(),font);
    }
    
    public void setZoom(float zoom) {
	view.setZoom(zoom);
    }

    public void getFocus() {
	view.getSelected().requestFocus();
    }

    //
    // General
    //

    public void showPopupMenu(Component c, int x, int y) {
	popupMenu.show(c,x,y);
    }

    public void saveXML(File file) {
	model.saveXML(file);
    }


    public void loadXML(File file) {
	model.loadXML(file);
	view.init();
    }

    //
    // Inner Classes
    //

    class QuitAction extends AbstractAction {
	String[] options = {"Yes","No","Cancel"};
	QuitAction() {
	    super("Quit");
	}
	public void actionPerformed(ActionEvent e) {
	    if (model.isSaved()) System.exit(0);
 
	    int returnVal = JOptionPane.showOptionDialog( view,"There are unsaved Mindmaps.\n Do you want to save them?", "Save before quit?",JOptionPane.YES_NO_CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE,null,options,options[0]);
	    if (returnVal==JOptionPane.YES_OPTION) {
		save.actionPerformed(e);
		System.exit(0);
	    } else if (returnVal==JOptionPane.NO_OPTION) {
		System.exit(0);
	    } else if (returnVal==JOptionPane.CANCEL_OPTION) {
		return;
	    }
	}
    }

    class NewMapAction extends AbstractAction {
	NewMapAction(Object controller) {
	    super("New", new ImageIcon(controller.getClass().getResource("/images/New24.gif")));
	    //Workaround to get the images loaded in jar file.
	    //they have to be added to jar manually with full path from root
	    //I really don't like this, but it's a bug of java
	}
	public void actionPerformed(ActionEvent e) {
	    model.newMap();
	    System.out.println("New MindMap");
	}
    }

    class SaveAction extends AbstractAction {
	SaveAction(Object controller) {
	    super("Save", new ImageIcon(controller.getClass().getResource("/images/Save24.gif")));
	}
	public void actionPerformed(ActionEvent e) {
	    if (model.isSaved()) return;
	    if (model.getFile()==null) {
		saveAs.actionPerformed(e);
	    } else {
		saveXML(model.getFile());
	    }
	}
    }
	    

    class SaveAsAction extends AbstractAction {
	SaveAsAction(Object controller) {
	    super("Save As", new ImageIcon(controller.getClass().getResource("/images/SaveAs24.gif")));
	}
	public void actionPerformed(ActionEvent e) {
	    JFileChooser chooser = new JFileChooser();
	    chooser.addChoosableFileFilter(new MindMapFilter());
	    int returnVal = chooser.showSaveDialog(view);
	    if (returnVal==JFileChooser.APPROVE_OPTION) {//ok pressed
		File f = chooser.getSelectedFile();
		//Force the extension to be .mm
		String ext = Tools.getExtension(f);
		if(!ext.equals("mm")) {
		    f = new File(f.getParent(),f.getName()+".mm");
		}
		saveXML(f);
	    }
	}
    }

    class OpenAction extends AbstractAction {
	String[] options = {"Yes","No","Cancel"};
	OpenAction(Object controller) {
	    super("Open", new ImageIcon(controller.getClass().getResource("/images/Open24.gif")));
	}
	public void actionPerformed(ActionEvent e) {
	    //This shouldn't be necessary if one can open several mindmaps at one time
	    if (!model.isSaved()) {
		int returnVal = JOptionPane.showOptionDialog( view,"There are unsaved Mindmaps.\n Do you want to save them?", "Save before quit?",JOptionPane.YES_NO_CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE,null,options,options[0]);
		if (returnVal==JOptionPane.YES_OPTION) {
		    save.actionPerformed(e);
		} else if (returnVal==JOptionPane.NO_OPTION) {
		} else if (returnVal==JOptionPane.CANCEL_OPTION) {
		    return;
		}
	    }
	    JFileChooser chooser = new JFileChooser();
	    chooser.addChoosableFileFilter(new MindMapFilter());
	    int returnVal = chooser.showOpenDialog(view);
	    if (returnVal==JFileChooser.APPROVE_OPTION) {
		loadXML(chooser.getSelectedFile());
	    }
	}
    }

    class CutAction extends AbstractAction {
	CutAction(Object controller) {
	    super("Cut", new ImageIcon(controller.getClass().getResource("/images/Cut24.gif")));
	}
	public void actionPerformed(ActionEvent e) {
	    MindMapNode node = view.getSelected().getModel();
	    if (node.isRoot()) return;
	    paste.setEnabled(true);
	    model.cut(node);
	}
    }

    class PasteAction extends AbstractAction {
	PasteAction(Object controller) {
	    super("Paste",new ImageIcon(controller.getClass().getResource("/images/Paste24.gif")));
	    setEnabled(false);
	}
	public void actionPerformed(ActionEvent e) {
	    setEnabled(false);
	    model.paste(view.getSelected().getModel());
	}
    }

    class ForkAction extends AbstractAction {
	ForkAction() {
	    super("Fork");
	}
	public void actionPerformed(ActionEvent e) {
	    model.setNodeStyle(view.getSelected().getModel(), "fork");
	}
    }

    class BubbleAction extends AbstractAction {
	BubbleAction() {
	    super("Bubble");
	}
	public void actionPerformed(ActionEvent e) {
	    model.setNodeStyle(view.getSelected().getModel(), "bubble");
	}
    }

    class LinearAction extends AbstractAction {
	LinearAction() {
	    super("Linear");
	}
	public void actionPerformed(ActionEvent e) {
	    model.setEdgeStyle(view.getSelected().getModel(), "linear");
	}
    }

    class BezierAction extends AbstractAction {
	BezierAction() {
	    super("Bezier");
	}
	public void actionPerformed(ActionEvent e) {
	    model.setEdgeStyle(view.getSelected().getModel(), "bezier");
	}
    }

    class ItalicAction extends AbstractAction {
	ItalicAction(Object controller) {
	    super("Italic", new ImageIcon(controller.getClass().getResource("/images/Italic24.gif")));
	}
	public void actionPerformed(ActionEvent e) {
	    model.setItalic(view.getSelected().getModel());
	}
    }

    class BoldAction extends AbstractAction {
	BoldAction(Object controller) {
	    super("Bold", new ImageIcon(controller.getClass().getResource("/images/Bold24.gif")));
	}
	public void actionPerformed(ActionEvent e) {
	    model.setBold(view.getSelected().getModel());
	}
    }

    class UnderlinedAction extends AbstractAction {
	UnderlinedAction(Object controller) {
	    super("Underline", new ImageIcon(controller.getClass().getResource("/images/Underline24.gif")));
	}
	public void actionPerformed(ActionEvent e) {
	    model.setUnderlined(view.getSelected().getModel());
	}
    }

    class NormalFontAction extends AbstractAction {
	NormalFontAction(Object controller) {
	    super("Normal", new ImageIcon(controller.getClass().getResource("/images/Normal24.gif")));	}
	public void actionPerformed(ActionEvent e) {
	    model.setNormalFont(view.getSelected().getModel());
	}
    }

    class NodeColorAction extends AbstractAction {
	NodeColorAction() {
	    super("Node Color");
	}
	public void actionPerformed(ActionEvent e) {
	    Color color = JColorChooser.showDialog(view,"Choose Node Color:",model.getNodeColor(view.getSelected().getModel()) );
	    model.setNodeColor(view.getSelected().getModel(), color);
	}
    }

    class EdgeColorAction extends AbstractAction {
	EdgeColorAction() {
	    super("Edge Color");
	}
	public void actionPerformed(ActionEvent e) {
	    MindMapNode node = view.getSelected().getModel();
	    	Color color = JColorChooser.showDialog(view,"Choose Edge Color:",model.getEdgeColor(node));
		model.setEdgeColor(node,color);
	}
    }

    class BackgroundAction extends AbstractAction {
	BackgroundAction() {
	    super("Background");
	}
	public void actionPerformed(ActionEvent e) {
	    Color color = JColorChooser.showDialog(view,"Choose Background Color:",view.getBackground() );
	    view.setBackground(color);
	}
    }


    class AboutAction extends AbstractAction {
	AboutAction() {
	    super("About");
	}
	public void actionPerformed(ActionEvent e) {
	    JOptionPane.showMessageDialog(view,"FreeMind\nThis Program is written by Joerg Mueller <joergmueller@bigfoot.com>\nin the hope that it will be useful.\nHave Fun with it! (It's under the GPL) \n Version: "+FreeMind.version);
	}
    }

    class MindMapFilter extends FileFilter {
	public boolean accept(File f) {
	    if (f.isDirectory()) return true;
	    String extension = Tools.getExtension(f);
	    if (extension != null) {
		if (extension.equals("mm")) {
		    return true;
		} else {
		    return false;
		}
	    }
	    return false;
	}
	
	public String getDescription() {
	    return "Mindmaps";
	}
    }

    /**For debugging purposes*/
    public class FocusListenerImpl implements FocusListener {
	public void focusGained(FocusEvent e) {
	}

	public void focusLost(FocusEvent e) {
	}
    }
}
