/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package freemind.main;

import javax.swing.JScrollPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JButton;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import freemind.model.MindMap;
import freemind.model.simplemodel.MapModel;
import freemind.model.simplemodel.NodeModel;
import freemind.view.mindmapview.MapView;
import freemind.controller.TopLevelKeyListener;
import freemind.controller.MenuBar;
import freemind.controller.Controller;
import freemind.controller.PopupMenu;
import freemind.controller.MainToolBar;
import freemind.controller.FontToolBar;
import javax.swing.JToolBar;

public class FreeMind extends JFrame {
    public static final String version = "0.0.2 build 14";
	//    private String lineStyle = "Angled"; 
    private MapView view;
    Controller c;
   
    public FreeMind() {
        super("FreeMind");

	MindMap model = sampleModel();

	c = new Controller();

        view = new MapView( model,c );

	c.setModel(model);
	c.setView(view);
	c.setFrame(this);

	//Create the MenuBar
	MenuBar menuBar;
	menuBar = new MenuBar(c);
	setJMenuBar(menuBar);

	//Add the KeyListener 
	addKeyListener( new TopLevelKeyListener( view ) );
	
	//Layout everything
	getContentPane().setLayout( new BorderLayout() );
	
	//Create the ToolBar
	JToolBar mainToolBar = new MainToolBar(c);
	getContentPane().add( mainToolBar, BorderLayout.NORTH );
	JToolBar fontToolBar = new FontToolBar(c);
	mainToolBar.add(fontToolBar);

        //Create the scroll pane and add the tree to it. 
	JScrollPane scrollPane = new JScrollPane( view );
	scrollPane.setPreferredSize( new Dimension( 600, 400 ) );
	
	getContentPane().add( scrollPane, BorderLayout.CENTER );

	//this would be nice but it doesn't work (too early)
	view.moveToRoot();

	//Disable the default close button, instead use windowListener
	setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    c.quit.actionPerformed(new ActionEvent(this,0,"quit"));
		}		
		public void windowActivated(WindowEvent e) {
		    //This doesn't work the first time, it's called too early to get Focus
		    view.getSelected().requestFocus();
		}
        });  
 

    }//Constructor

    /**
     * For testing purposes
     */
    private MapModel sampleModel() {
	NodeModel root = new NodeModel("FreeMind");
	MapModel view = new MapModel( root );
	NodeModel keyboard = new NodeModel("Keyboard");
	NodeModel navigation = new NodeModel("Navigation");
	NodeModel cursor = new NodeModel("Cursor Keys");
	NodeModel emacs = new NodeModel("Emacs-Style");
	NodeModel previous = new NodeModel("C-P = Previous");
	NodeModel next = new NodeModel("C-N = Next");
	NodeModel forward = new NodeModel("C-F = Forward");
	NodeModel back = new NodeModel("C-B = Back");
	NodeModel editing = new NodeModel("Editing");
	NodeModel insert = new NodeModel("Insert");
	NodeModel ins = new NodeModel("INS-Key");
	NodeModel mn = new NodeModel("M-N");
	NodeModel delete = new NodeModel("Delete");
	NodeModel del = new NodeModel("DEL-Key");
	NodeModel md = new NodeModel("M-D");
	NodeModel explanation = new NodeModel("Explanation");
	NodeModel ctrl = new NodeModel("C = Control");
	NodeModel alt = new NodeModel("M = Alt");
	NodeModel mouse = new NodeModel("Mouse");
	NodeModel popup = new NodeModel("Use the Popup-Menu");

	root.add(keyboard);
	keyboard.add(navigation);
	navigation.add(cursor);
	navigation.add(emacs);
	emacs.add(previous);
	emacs.add(next);
	emacs.add(forward);
	emacs.add(back);
	keyboard.add(editing);
	editing.add(insert);
	insert.add(ins);
	insert.add(mn);
	editing.add(delete);
	delete.add(del);
	delete.add(md);
	root.add(explanation);
	explanation.add(ctrl);
	explanation.add(alt);
	root.add(mouse);
	mouse.add(popup);

	return view;
    }

    public static void main(String[] args) {
        JFrame frame = new FreeMind();
 
        frame.pack();
        frame.setVisible(true);

    }//main()
}
