/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package freemind.model;

import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.MutableTreeNode;
import java.awt.Color;
import java.io.File;

public interface MindMap extends TreeModel {

    void newMap();

    void saveXML(File file);
        
    void loadXML(File file);

    boolean isSaved();
        
    File getFile();

    void setEdgeColor(MindMapNode node, Color color);

    Color getEdgeColor(MindMapNode node);

    void setEdgeStyle(MindMapNode node, String style);

    String getEdgeStyle(MindMapNode node);

    void setNodeStyle(MindMapNode node, String style);

    String getNodeStyle(MindMapNode node);

    void setNodeColor(MindMapNode node, Color color);

    Color getNodeColor(MindMapNode node);

    boolean isBold(MindMapNode node);

    void setBold(MindMapNode node);

    boolean isItalic(MindMapNode node);

    void setItalic(MindMapNode node);

    boolean isUnderlined(MindMapNode node);

    void setUnderlined(MindMapNode node);

    void setNormalFont(MindMapNode node);

    int getFontSize(MindMapNode node);

    void setFontSize(MindMapNode node, int fontSize);

    String getFont(MindMapNode node);

    void setFont(MindMapNode node, String font);

    Object[] getPathToRoot( TreeNode node );

    void changeNode(MindMapNode node, String newText);

    void cut(MindMapNode node);

    void paste(MindMapNode parent);

    /**
     * Use this method to add children because it will cause the appropriate event.
     */
     void insertNodeInto(MutableTreeNode newChild,
			 MutableTreeNode parent, int index);
    
    /**
     * Message this to remove node from its parent. This will message
     * nodesWereRemoved to create the appropriate event. This is the
     * preferred way to remove a node as it handles the event creation
     * for you.
     */
    void removeNodeFromParent(MutableTreeNode node);
    
}
































































