/*FreeMind - A Program for creating and viewing Mindmaps
 *Copyright (C) 2000  Joerg Mueller <joergmueller@bigfoot.com>
 *See COPYING for Details
 *
 *This program is free software; you can redistribute it and/or
 *modify it under the terms of the GNU General Public License
 *as published by the Free Software Foundation; either version 2
 *of the License, or (at your option) any later version.
 *
 *This program is distributed in the hope that it will be useful,
 *but WITHOUT ANY WARRANTY; without even the implied warranty of
 *MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *GNU General Public License for more details.
 *
 *You should have received a copy of the GNU General Public License
 *along with this program; if not, write to the Free Software
 *Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package freemind.view.simpleview;

import freemind.model.MindMap;
import freemind.model.MindMapNode;
import freemind.model.simplemodel.MMNode;
import freemind.controller.TreeKeyListener;
import javax.swing.JTree;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.MutableTreeNode;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;

/**
 * A simple implementation of the view. Maybe editing should be moved to
 * a controller package soon.
 */
public class MMTree extends JTree {
    
    private final String lineStyle = "Angled";
    MindMap treeModel;
    MindMapNode rootNode;

    //
    // Constructors
    //

    public MMTree( MindMap model ) {
	super( model );
	treeModel = model;
	addKeyListener(new TreeKeyListener(this));
	rootNode = (MindMapNode)treeModel.getRoot();
	getSelectionModel().setSelectionMode( TreeSelectionModel.SINGLE_TREE_SELECTION );
	putClientProperty( "JTree.lineStyle", lineStyle );
    }
	
    
    //
    // Methods for editing
    //

    /**Remove the currently selected node. */
    public void removeCurrentNode() {
	TreePath currentSelection = getSelectionPath();
	if (currentSelection != null) {
	    MutableTreeNode currentNode = (MutableTreeNode)currentSelection.getLastPathComponent();
	    MutableTreeNode parent = (MutableTreeNode)(currentNode.getParent());
	    if (parent != null) {
		( (MindMap)treeModel ).removeNodeFromParent(currentNode);
		return;
	    }
	}//Either there was no selection, or root was selected.
    } 

    public void addNew() {
	startEditingAtPath( ( (MMNode)addObject( "New Node" ) ).getPath() );
	//obfuscated ;-) . Creates a new Node and starts editing.
	//doesn't work without text in the String: something like addObject("") or addObject(".") produces a bug.
    }

    /**Add child to the currently selected node. */
    public MutableTreeNode addObject( Object child ) {
	MutableTreeNode parentNode = null;
	TreePath parentPath = getSelectionPath();

	if (parentPath == null) {
	    parentNode = rootNode;
	} else {
	    parentNode = (MutableTreeNode)( parentPath.getLastPathComponent() );
	}
	return addObject( parentNode, child, true );
    }

    public MutableTreeNode addObject( MutableTreeNode parent, Object child ) {
	return addObject( parent, child, false );
    }

    public MutableTreeNode addObject( MutableTreeNode parent,
				      Object child,
				      boolean shouldBeVisible) {
	MutableTreeNode childNode = new MMNode( child );
	//Not sure if this way works for us (change it in future)
	if (parent == null) {
	    parent = rootNode;
	}

	treeModel.insertNodeInto( childNode, parent, parent.getChildCount() );

	if (shouldBeVisible) {
	    scrollPathToVisible( new TreePath( ( (MMNode)childNode ).getPath() ) );
	}

	return childNode;

    }//addObject()

}












